function [ X ] = pinvGreville( A, tol )
% Moore-Penroseova inverze za vyuit Grevillova algoritmu
%
%   [ X ] = pinvGreville( A, tol )
%   Vstup:  A ... matice m x n
%           tol ... tolerance pro rozpoznn nulovho vektoru
%   Vstup: X ... Moore-Penroseova inverze matice A
%
%   Datum posledn pravy: 8.8.2015, KF

[m, n] = size(A);
% Pro prvn sloupec matice A
c = A(:, 1);
if norm(c, inf) <= tol
    % Prvky vektoru jsou zanedbateln mal
    % a jsou tedy povaovny za nulov
    X = zeros(1, m);
else
    X = c' / (c' * c);
end
% Pro druh a n-t sloupec
for j = 2 : n
    d = X * A(:, j);
    c = A(:, j) - A(:, 1 : j - 1) * d;
    if norm(c, inf) <= tol
        % Prvky vektoru jsou zanedbateln mal
        % a jsou tedy povaovny za nulov
        bt = (d' * X) / (1 + d' * d);
    else
        bt = c' / (c' * c);
    end
    % Grevillv vzorec
    X = [X - d * bt; bt];
end
end

