function [ X, A ] = spektRozklad( A, tol )
% Spektrln rozklad matice pomoc Jacobiho metody 
%
%   [ X, L ] = spektRozklad( A, tol )
%   Vstup:  A ... symetrick matice n x n
%           tol ... tolerance udvajc poadovanou pesnost vsledk
%   Vstup: L ... diagonln matice n x n, jej diagonln prvky l_ii 
%                 aproximuj vlastn sla lambda_i matice A, 
%                 abs(lambda_i - l_ii) <= tol * norm(A, 'fro'), i = 1,...,n
%           X ... ortogonln matice n x n, jej sloupce jsou tvoeny
%                 pslunmi vlastnmi vektory matice A,
%                 A = X * L * X'
%
%   Datum posledn pravy: 1.11.2015, KF

[n, n2] = size(A);
if n ~= n2
    error ('Zadan matice nen tvercov.')
end
if ~issymmetric(A)
    error ('Zadan matice nen symetrick.')
end
X = eye(n);
delta = tol * norm(A, 'fro');
while off(A) > delta
    % Nalezen maxima z nediagonlnch prvk
    p = 1;
    q = 2;
    for i = 1 : n
        for j = i + 1 : n
            if abs(A(i, j)) > abs(A(p, q))
                p = i;
                q = j;
            end
        end
    end
    % Vpoet hodnot sinus (s) a kosinus (c)
    if A(p, q) ~= 0
        tau = (A(q, q) - A(p, p)) / (2 * A(p, q));
        if tau >= 0
            t = 1 / (tau + sqrt(1 + tau ^ 2));
        else
            t = -1 / (-tau + sqrt(1 + tau ^ 2));
        end
        c = 1 / sqrt(1 + t ^ 2);
        s = t * c;
    else
        c = 1;
        s = 0;
    end
    % Aktualizace matic A (A = G' * A * G) a  X (X = X * G)
    A([p q], :) = [c s; -s c]' * A([p q], :);
    A(:, [p q]) = A(:, [p q]) * [c s; -s c];
    X(:, [p q]) = X(:, [p q]) * [c s; -s c];
end
% Seazen diagonlnch prvk vsledn matice A
[b, p] = sort(diag(A), 'descend');
A = diag(b);
X = X(:, p);
end

